from grayskull.cli.parser import parse_pkg_name_version


def test_parse_pkg_name_version():
    pkg_name, pkg_version = parse_pkg_name_version("pytest=5.3.5")
    assert pkg_name == "pytest"
    assert pkg_version == "5.3.5"

    pkg_name, pkg_version = parse_pkg_name_version("pytest==5.3.5")
    assert pkg_name == "pytest"
    assert pkg_version == "5.3.5"

    pkg_name, pkg_version = parse_pkg_name_version("pytest")
    assert pkg_name == "pytest"
    assert pkg_version is None



from pathlib import Path

import pytest

from grayskull.base.track_packages import (
    ConfigPkg,
    _get_track_info_from_file,
    _version_solver,
    parse_delimiter,
    solve_list_pkg_name,
    solve_pkg_name,
    solve_version_delimiter,
    track_package,
)


@pytest.fixture
def path_example() -> Path:
    return Path(__file__).parent.parent / "data" / "track_package" / "example.yaml"


def test_config_pkg():
    cfg = ConfigPkg("name", "import_name", "conda_forge", "min", "max")
    assert cfg.name == "name"
    assert cfg.import_name == "import_name"
    assert cfg.conda_forge == "conda_forge"
    assert cfg.delimiter_min == "min"
    assert cfg.delimiter_max == "max"

    cfg = ConfigPkg("name")
    assert cfg.name == "name"
    assert cfg.import_name == "name"
    assert cfg.conda_forge == "name"
    assert cfg.delimiter_min == ""
    assert cfg.delimiter_max == ""


def test_track_package(path_example: Path):
    foo_pkg = track_package("foo_pkg", path_example)
    assert foo_pkg.name == "foo_pkg"
    assert foo_pkg.import_name == "foo_import"
    assert foo_pkg.conda_forge == "foo_conda_forge"
    assert foo_pkg.delimiter_min == "1.2.3"
    assert foo_pkg.delimiter_max == "2.1.0"

    bar_pkg = track_package("bar_pkg", path_example)
    assert bar_pkg.name == "bar_pkg"
    assert bar_pkg.import_name == "bar_import"
    assert bar_pkg.conda_forge == "bar_conda_forge"
    assert bar_pkg.delimiter_min == ""
    assert bar_pkg.delimiter_max == ""

    foo_bar_pkg = track_package("foo_bar", path_example)
    assert foo_bar_pkg.name == "foo_bar"
    assert foo_bar_pkg.import_name == "foo_bar"
    assert foo_bar_pkg.conda_forge == "foo_bar_cf"
    assert foo_bar_pkg.delimiter_min == ""
    assert foo_bar_pkg.delimiter_max == ""

    no_pkg = track_package("NO_PKG", path_example)
    assert no_pkg.name == "NO_PKG"
    assert no_pkg.import_name == "NO_PKG"
    assert no_pkg.conda_forge == "NO_PKG"
    assert no_pkg.delimiter_min == ""
    assert no_pkg.delimiter_max == ""


def test_get_track_info_from_file(path_example):
    dict_exp = {
        "foo_pkg": {
            "import_name": "foo_import",
            "conda_forge": "foo_conda_forge",
            "delimiter_min": "1.2.3",
            "delimiter_max": "2.1.0",
        },
        "bar_pkg": {"import_name": "bar_import", "conda_forge": "bar_conda_forge"},
        "foo_bar": {"conda_forge": "foo_bar_cf"},
    }
    assert _get_track_info_from_file(path_example) == dict_exp
    assert _get_track_info_from_file(str(path_example)) == dict_exp


def test_parse_delimiter():
    assert parse_delimiter("<1.0.0,>2.0.0") == [("<", "1.0.0"), (">", "2.0.0")]
    assert parse_delimiter("<=1.0.0,>=2.0.0") == [("<=", "1.0.0"), (">=", "2.0.0")]
    assert parse_delimiter("!=1.0.0, >2.0.0") == [("!=", "1.0.0"), (">", "2.0.0")]
    assert parse_delimiter("==2.0.0") == [("==", "2.0.0")]


def test_solve_version_delimiter():
    assert (
        solve_version_delimiter(
            "pkg >=1.5.0,<1.8.0",
            ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
        )
        == ">=1.5.0,<1.8.0"
    )
    assert (
        solve_version_delimiter(
            ">0.5.0,<=1.8.0",
            ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
        )
        == ">=1.0.0,<=1.8.0"
    )
    assert (
        solve_version_delimiter(
            ">0.5.0,<2.5.0",
            ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
        )
        == ">=1.0.0,<2.0.0"
    )
    assert (
        solve_version_delimiter(
            "1.2.3", ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0")
        )
        == "1.2.3"
    )
    assert (
        solve_version_delimiter(
            ">=1.5.0,<2.5.0,!=1.6.3",
            ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
        )
        == ">=1.5.0,<2.0.0,!=1.6.3"
    )


def test_version_solver():
    assert _version_solver(
        [(">=", "1.5.0"), ("<", "1.8.0")],
        ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
    ) == [">=1.5.0", "<1.8.0"]
    assert _version_solver(
        [(">", "0.5.0"), ("<=", "1.8.0")],
        ConfigPkg("foo", delimiter_min="1.0.0", delimiter_max="2.0.0"),
    ) == [">=1.0.0", "<=1.8.0"]


def test_solve_pkg_name(path_example):
    assert (
        solve_pkg_name("foo_pkg >1.5.0,<=2.5.0", path_example)
        == "foo_conda_forge >1.5.0,<2.1.0"
    )
    assert solve_pkg_name("normal_pkg", path_example) == "normal_pkg"


def test_solve_list_pkg_name(path_example):
    assert solve_list_pkg_name(
        ["foo_pkg >1.5.0,<=2.5.0", "normal_pkg", "bar_pkg"], path_example
    ) == ["foo_conda_forge >1.5.0,<2.1.0", "normal_pkg", "bar_conda_forge"]



from ruamel.yaml.comments import CommentedMap

from grayskull.base.section import Section


def test_section_yaml_obj():
    commented_map = CommentedMap({"section1": {"subsection1": {"subsection2": "item"}}})
    sec = Section("MAIN_SECTION", parent_yaml=commented_map)
    assert sec.yaml_obj == commented_map.update({"MAIN_SECTION": None})


def test_add_subsection():
    sec = Section("MAIN_SEC")
    sec.add_subsection("SUBSECTION")
    assert "SUBSECTION" in sec
    assert sec.section_name == "MAIN_SEC"


def test_add_item():
    sec = Section("MAIN_SEC")
    sec.add_item("pkg1")
    sec.add_item("pkg2  # [win]")
    sec.add_item("pkg3")
    assert "pkg1" in sec
    assert "pkg2" in sec
    assert "pkg3" in sec


def test_section_children():
    commented_map = CommentedMap(
        {
            "section1": CommentedMap(
                {"subsection1": CommentedMap({"subsection2": "item"})}
            )
        }
    )
    sec = Section("section1", parent_yaml=commented_map)
    assert sec.section_name == "section1"
    assert sec.values[0].section_name == "subsection1"


def test_section_load():
    commented_map = CommentedMap({"section1": {"subsection1": {"subsection2": "item"}}})
    sec = Section("section1", commented_map)
    assert sec.section_name == "section1"
    assert sec["subsection1"].section_name == "subsection1"
    assert sec["subsection1"]["subsection2"].section_name == "subsection2"
    assert sec["subsection1"]["subsection2"][0].value == "item"


def test_repr():
    commented_map = CommentedMap({"section1": {"subsection1": {"subsection2": "item"}}})
    sec = Section("section1", commented_map)
    assert repr(sec) == "Section(section_name=section1, subsection=['subsection1'])"


def test_str():
    commented_map = CommentedMap()
    sec = Section("section1", commented_map)
    assert str(sec) == "section1"


def test_reduce_section():
    commented_map = CommentedMap({})
    sec = Section("section", commented_map)
    sec.add_items(["item", None])
    sec.values[0].selector = "# [win]"
    assert sec.values == ["item", None]
    assert sec._get_parent()[sec.section_name] == ["item", None]

    sec.reduce_section()
    assert sec._get_parent()[sec.section_name] == "item"
    assert sec.values == ["item"]


def test_hash():
    commented_map = CommentedMap()
    sec = Section("section1", commented_map)
    assert hash(sec) == hash("section1-[]")
    sec.add_item("item")
    assert hash(sec) == hash("section1-['item']")



import os

from grayskull.utils import (
    get_all_modules_imported_script,
    get_std_modules,
    get_vendored_dependencies,
)


def test_get_std_modules():
    std_modules = get_std_modules()
    assert "sys" in std_modules
    assert "os" in std_modules
    assert "ast" in std_modules
    assert "typing" in std_modules


def test_get_all_modules_imported_script(data_dir):
    all_imports = get_all_modules_imported_script(
        os.path.join(data_dir, "foo_imports.py")
    )
    assert sorted(all_imports) == sorted(["numpy", "pandas", "requests", "os", "sys"])


def test_get_vendored_dependencies(data_dir):
    all_deps = get_vendored_dependencies(os.path.join(data_dir, "foo_imports.py"))
    assert sorted(all_deps) == sorted(["numpy", "pandas", "requests"])



import ast
import os
from functools import lru_cache
from glob import glob
from typing import List


@lru_cache(maxsize=10)
def get_std_modules() -> List:
    from stdlib_list import stdlib_list

    all_libs = set()
    for py_ver in ("2.7", "3.6", "3.7", "3.8"):
        all_libs.update(stdlib_list(py_ver))
    return list(all_libs)


def get_all_modules_imported_script(script_file: str) -> set:
    modules = set()

    def visit_Import(node):
        for name in node.names:
            if name.name:
                modules.add(name.name.split(".")[0])

    def visit_ImportFrom(node):
        # if node.module is missing it's a "from . import ..." statement
        # if level > 0 it's a "from .submodule import ..." statement
        if node.module is not None and node.level == 0:
            if node.module:
                modules.add(node.module.split(".")[0])

    node_iter = ast.NodeVisitor()
    node_iter.visit_Import = visit_Import
    node_iter.visit_ImportFrom = visit_ImportFrom
    with open(script_file, "r") as f:
        node_iter.visit(ast.parse(f.read()))
    return modules


def get_vendored_dependencies(script_file: str) -> List:
    """Get all third part dependencies which are being in use in the setup.py

    :param script_file: Path to the setup.py
    :return: List with all vendored dependencies
    """
    all_std_modules = get_std_modules()
    all_modules_used = get_all_modules_imported_script(script_file)
    local_modules = get_local_modules(os.path.dirname(script_file))
    vendored_modules = []
    for dep in all_modules_used:
        if dep in local_modules or dep in all_std_modules:
            continue
        vendored_modules.append(dep.lower())
    return vendored_modules


@lru_cache(maxsize=20)
def get_local_modules(sdist_folder: str) -> List:
    result = []
    for py_file in glob(f"{sdist_folder}/*.py"):
        py_file = os.path.basename(py_file)
        if py_file == "setup.py":
            continue
        result.append(os.path.splitext(py_file)[0])
    return result



from typing import Optional

import progressbar

WIDGET_BAR_DOWNLOAD = [
    progressbar.Percentage(),
    " ",
    progressbar.ETA(format="%(eta)8s"),
    " ",
    progressbar.AdaptiveTransferSpeed(),
    progressbar.Bar(),
]


class CLIConfig:
    __instance: Optional["CLIConfig"] = None

    def __new__(cls, stdout: bool = False, list_missing_deps: bool = False):
        if CLIConfig.__instance is None:
            CLIConfig.__instance = object.__new__(cls)
            CLIConfig.__instance.stdout = stdout
            CLIConfig.__instance.list_missing_deps = list_missing_deps
        return CLIConfig.__instance



import weakref
from typing import Any, Iterator, List, Optional, Union

from ruamel.yaml.comments import CommentedMap, CommentedSeq

from grayskull.base.recipe_item import RecipeItem


class Section:
    def __init__(self, section_name: str, parent_yaml: Optional[CommentedMap] = None):
        self._section_name = section_name.strip()
        if parent_yaml is None:
            self.__parent = CommentedMap()
            if section_name not in self.__parent:
                self.__parent[section_name] = None
        else:
            if section_name not in parent_yaml:
                parent_yaml[section_name] = None
            self.__parent = weakref.ref(parent_yaml)

    @property
    def values(self) -> List:
        parent = self._get_parent()
        result = []
        if isinstance(self.yaml_obj, CommentedMap):
            result = [Section(name, self.yaml_obj) for name in self.yaml_obj.keys()]
        elif isinstance(self.yaml_obj, dict):
            parent[self.section_name] = CommentedMap(self.yaml_obj)
            result = [Section(name, self.yaml_obj) for name in self.yaml_obj.keys()]
        elif isinstance(self.yaml_obj, CommentedSeq):
            result = [
                RecipeItem(pos, self.yaml_obj) for pos in range(len(self.yaml_obj))
            ]
        elif self.yaml_obj is not None:
            parent[self.section_name] = CommentedSeq([self.yaml_obj])
            result = [RecipeItem(0, parent[self.section_name])]
        return result

    @property
    def section_name(self) -> str:
        return self._section_name

    def _get_parent(self):
        if isinstance(self.__parent, CommentedMap):
            return self.__parent
        return self.__parent()

    @property
    def yaml_obj(self) -> Union[CommentedMap, CommentedSeq]:
        """Get of the yaml object which is being handled for this section.
        Please do not modify this object directly unless you know what you are
        doing.
        """
        parent = self._get_parent()
        if self.section_name not in parent:
            self.add_subsection(self.section_name)
        return parent[self.section_name]

    def reduce_section(self):
        if not self.values:
            return
        if isinstance(self.values[0], Section):
            for section in self.values:
                section.reduce_section()
        elif isinstance(self.values[0], RecipeItem):
            for recipe_item in self.values:
                if recipe_item.value is None:
                    self.yaml_obj.pop(self.yaml_obj.index(recipe_item.value))
            if len(self.yaml_obj) == 1:
                val = self._get_parent()[self.section_name][0]
                if self._get_parent()[self.section_name].ca.items:
                    comment_token = self.yaml_obj.ca.items.get(0)
                    if comment_token:
                        comment_token[0].value = f" {comment_token[0].value}"
                        self._get_parent().ca.items[self.section_name] = [
                            None,
                            None,
                            comment_token[0],
                            None,
                        ]
                self._get_parent()[self.section_name] = val

    def __hash__(self) -> int:
        return hash(f"{self}-{[str(v) for v in self.values]}")

    def __str__(self) -> str:
        return self.section_name

    def __repr__(self) -> str:
        val = ""
        if isinstance(self.yaml_obj, (CommentedMap, dict)):
            val = f"subsection={list(self.yaml_obj.keys())}"
        elif self.yaml_obj is not None:
            val = f"items={self.yaml_obj}"
        if val:
            val = f", {val}"
        return f"Section(section_name={self._section_name}{val})"

    def __len__(self) -> int:
        if self.yaml_obj:
            return len(self.yaml_obj)
        return 0

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, Section):
            return other.yaml_obj == self.yaml_obj
        if len(self.values) == 1:
            return self.values[0] == other or str(self.section_name) == str(other)
        if isinstance(other, str):
            return str(self.section_name) == str(other)
        if isinstance(other, list) and isinstance(other[0], str):
            for pos, item in enumerate(self.values):
                if item.value != other[pos] and str(item.value) != other[pos]:
                    return False
            return True
        return other == self.values

    def __iter__(self) -> Iterator:
        if not self.yaml_obj:
            return iter([])
        return iter(self.values)

    def __getitem__(self, item: Union[str, int]) -> Union["Section", RecipeItem, None]:
        if isinstance(item, str):
            for child in self.values:
                if child.section_name == item:
                    return child
            return Section(item, parent_yaml=self.yaml_obj)
        return self.values[item]

    def __setitem__(self, key: str, value: Any):
        if key not in self.yaml_obj:
            self.add_subsection(key)
        if isinstance(value, (str, int)):
            self.yaml_obj[key] = CommentedSeq()
            self[key].add_item(value)
        elif isinstance(value, dict):
            Section(key, self.yaml_obj)

    def add_subsection(self, section: Union[str, "Section"]):
        """Add a subsection to the current Section. If the current section has a
        list of items or just an item, it will replace it by a subsection.

        :param section: Receives the name of a new subsection or a section object
        which will be populated as a child of the current section
        """
        if not isinstance(self.yaml_obj, CommentedMap):
            self._get_parent()[self.section_name] = CommentedMap()
        if isinstance(section, Section):
            self._get_parent()[section.section_name] = section.yaml_obj
        return Section(section, parent_yaml=self.yaml_obj)

    def add_item(self, item: Union[str, int]):
        """Add a new item to the current section

        :param item: Receive the value for the current item
        """
        if not isinstance(self.yaml_obj, CommentedSeq):
            self._get_parent()[self.section_name] = CommentedSeq()
        RecipeItem(len(self.yaml_obj), self.yaml_obj, item)

    def add_items(self, items: List):
        for item in items:
            self.add_item(item)



import logging
import subprocess

import requests
from colorama import Fore

from grayskull.cli.stdout import print_msg

log = logging.getLogger(__name__)


def get_git_current_user_metadata() -> dict:
    git_out = subprocess.check_output(["git", "config", "user.name"])
    return requests.get(
        url="https://api.github.com/search/users",
        params={"q": git_out.strip()},
        timeout=5,
    ).json()


def get_git_current_user() -> str:
    try:
        github_search = get_git_current_user_metadata()
        if github_search["total_count"] >= 1:
            return github_search["items"][0]["login"]
    except Exception as err:
        log.debug(
            f"Exception occurred when trying to recover user information from github."
            f" Exception: {err}"
        )
    print_msg(
        f"Using default recipe maintainer:"
        f" {Fore.LIGHTMAGENTA_EX}AddYourGitHubIdHere"
    )
    return "AddYourGitHubIdHere"



import re
from typing import Optional, Tuple


def parse_pkg_name_version(pkg_name: str) -> Tuple[str, Optional[str]]:
    pkg = re.match(r"([a-zA-Z0-9\-_\.]+)=+([a-zA-Z0-9\-_\.]+)", pkg_name)
    return pkg.groups() if pkg else (pkg_name, None)



from grayskull.__main__ import main


def test_loop_deps_nipy(tmpdir):
    out_folder = tmpdir.mkdir("out")
    main(["pypi", "nipy=0.4.2", "-o", str(out_folder), "--download"])
    nipy_folder = out_folder / "nipy"
    assert nipy_folder.isdir()

    recipe_file = nipy_folder / "meta.yaml"
    assert recipe_file.isfile()
    assert (nipy_folder / "pypi.json").isfile()
    assert (nipy_folder / "nipy-0.4.2.tar.gz").isfile()



import os

from pytest import fixture

from grayskull.base.base_recipe import AbstractRecipeModel


class EmptyGray(AbstractRecipeModel):
    def refresh_section(self, section="", **kwargs):
        if section == "source":
            self["source"]["sha256"] = "sha256_foo"
            self["source"]["url"] = "URL"
        elif section == "build":
            self["build"]["number"] = 1


@fixture
def data_recipes(data_dir: str) -> str:
    return os.path.join(data_dir, "recipes")


def test_update_all_recipe(data_recipes):
    recipe = EmptyGray(load_recipe=os.path.join(data_recipes, "simple_jinja_var.yaml"))
    recipe.update_all_recipe()
    assert recipe["build"]["number"] == 1
    assert recipe["source"]["sha256"] == "sha256_foo"
    assert recipe["source"]["url"] == "URL"


def test_refresh_section(data_recipes):
    recipe = EmptyGray(load_recipe=os.path.join(data_recipes, "simple_jinja_var.yaml"))
    assert "number" not in recipe["source"]

    recipe.refresh_section("build")
    assert recipe["build"]["number"].values[0] == 1
    assert "sha256" not in recipe["source"]
    assert "url" not in recipe["source"]

    recipe.refresh_section("source")
    assert recipe["build"]["number"] == 1
    assert recipe["source"]["sha256"] == "sha256_foo"
    assert recipe["source"]["url"] == "URL"


def test_generate_recipe(tmpdir, data_recipes):
    recipe = EmptyGray(name="pkg1", version="1.0.0")
    recipe.update_all_recipe()
    recipe.generate_recipe(tmpdir, mantainers=["marcelotrevisani"])

    with open(tmpdir / "pkg1" / "meta.yaml") as recipe_file:
        generated_recipe = recipe_file.read()
    with open(os.path.join(data_recipes, "empty_gray.yaml")) as recipe_file:
        exp_recipe = recipe_file.read()
    assert generated_recipe == exp_recipe


def test_jinja_var(data_recipes):
    recipe = EmptyGray(load_recipe=os.path.join(data_recipes, "simple_jinja_var.yaml"))
    assert recipe.get_jinja_var("name") == "foo_pkg-123.test"
    assert recipe.get_jinja_var("version") == "1.2.4"
    assert recipe["package"]["name"] == "<{ name|lower }}"
    assert recipe["package"]["version"] == "<{ version }}"

    recipe.set_jinja_var("version", "0.1.1")
    assert recipe.get_jinja_var("version") == "0.1.1"

    recipe.add_jinja_var("foo", "bar")
    assert recipe.get_jinja_var("foo") == "bar"

    recipe.set_jinja_var("bar", "foo")
    assert recipe.get_jinja_var("bar") == "foo"


def test_set_default_values():
    recipe = EmptyGray(name="PkgName", version="1.0.0")
    assert recipe.get_var_content(recipe["package"]["name"].values[0]) == "PkgName"
    assert recipe["package"]["name"].values[0] == "<{ name|lower }}"

    assert recipe["package"]["version"].values[0] == "<{ version }}"
    assert recipe.get_var_content(recipe["package"]["version"].values[0]) == "1.0.0"


def test_get_set_var_content():
    recipe = EmptyGray(name="PkgName", version="1.0.0")
    assert recipe.get_var_content(recipe["package"]["name"].values[0]) == "PkgName"
    assert recipe.get_var_content(recipe["package"]["version"].values[0]) == "1.0.0"

    recipe.set_var_content(recipe["package"]["version"].values[0], "2.1.3")
    assert recipe.get_var_content(recipe["package"]["version"].values[0]) == "2.1.3"
    assert recipe["package"]["version"].values[0] == "<{ version }}"


def test_getitem():
    pkg = EmptyGray("pytest", "5.0.0")
    assert pkg["build"].section_name == "build"
    assert pkg["build"]["number"].section_name == "number"
    assert pkg["build"]["number"][0] == 1
